<?php

namespace Drupal\migration_complexity_estimator\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\migration_complexity_estimator\Service\EntityAnalyzer;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;

/**
 * Controller for generating migration complexity reports.
 */
class ReportController extends ControllerBase {

  /**
   * The entity analyzer service.
   *
   * @var \Drupal\migration_complexity_estimator\Service\EntityAnalyzer
   */
  protected $entityAnalyzer;

  /**
   * The tempstore private service.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Constructs a new ReportController.
   *
   * @param \Drupal\migration_complexity_estimator\Service\EntityAnalyzer $entityAnalyzer
   *   The entity analyzer service.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore private service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager service.
   */
  public function __construct(
    EntityAnalyzer $entityAnalyzer,
    PrivateTempStoreFactory $temp_store_factory,
    FormBuilderInterface $form_builder,
    EntityTypeManagerInterface $entity_type_manager,
    EntityFieldManagerInterface $entity_field_manager,
  ) {
    $this->entityAnalyzer = $entityAnalyzer;
    $this->tempStoreFactory = $temp_store_factory;
    $this->formBuilder = $form_builder;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('migration_complexity_estimator.entity_analyzer'),
      $container->get('tempstore.private'),
      $container->get('form_builder'),
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * Generates the migration complexity report.
   *
   * @return array
   *   A render array for the report.
   */
  public function overview(): array {
    // Fetch stored filters from tempstore.
    $tempstore = $this->tempStoreFactory->get('migration_complexity_estimator');
    $filters = $tempstore->get('filters') ?? [];

    // Apply filter for unused status.
    // Default to 'all'.
    $unused_status = $filters['unused_status'] ?? 'all';

    $entity_types = $this->entityAnalyzer->getEntityTypesAndBundles();
    $grouped_report = [];

    // Initialize counters for the summary.
    $total_entity_types = 0;
    $total_bundles = 0;
    $total_fields = 0;
    $total_entities = 0;
    $unused_bundles = 0;
    $total_entity_references = 0;
    $total_files_image_fields = 0;
    $total_multi_value_fields = 0;

    foreach ($entity_types as $entity_type_id => $entity_type) {
      if (!empty($filters['entity_type']) && stripos($entity_type_id, $filters['entity_type']) === FALSE) {
        continue;
      }

      $has_bundles = FALSE;

      foreach ($entity_type['bundles'] as $bundle_id => $bundle) {
        if (!empty($filters['bundle']) && stripos($bundle_id, $filters['bundle']) === FALSE) {
          continue;
        }

        $label = $entity_type['label'] . ' (' . $bundle['label'] . ')';
        if (!empty($filters['label']) && stripos($label, $filters['label']) === FALSE) {
          continue;
        }

        // Filter by unused status.
        $entity_count = $this->entityAnalyzer->countEntities($entity_type_id, $bundle_id);
        if ($unused_status !== 'all') {
          // Check for bundles with no entities for unused status.
          if (($unused_status == 'unused' && $entity_count > 0) ||
              ($unused_status == 'used' && $entity_count == 0)) {
            continue;
          }
        }

        // Flag the bundle as unused if entity count is 0.
        $is_unused = ($entity_count === 0) ? 'Unused' : 'Used';
        $fields = $this->entityAnalyzer->analyzeFields($entity_type_id, $bundle_id);

        foreach ($fields as $field) {
          $field_type = $field['field_type'];

          if (in_array($field_type, ['entity_reference', 'entity_reference_revisions'])) {
            $total_entity_references++;
            $total_fields--;
          }

          if (in_array($field_type, ['file', 'image'])) {
            $total_files_image_fields++;
            $total_fields--;
          }

          if (!empty($field['cardinality']) && $field['cardinality'] > 1) {
            $total_multi_value_fields++;
            $total_fields--;
          }
        }

        $grouped_report[$entity_type_id]['label'] = $entity_type['label'];
        $grouped_report[$entity_type_id]['bundles'][] = [
          'bundle' => $bundle_id,
          'label' => $label,
          'entity_count' => $entity_count,
          'field_count' => count($fields),
          'fields' => $this->entityAnalyzer->formatFields($entity_type_id, $bundle_id),
          'status' => $is_unused,
        ];

        // Update counters.
        $total_bundles++;
        $total_fields += count($fields);
        $total_entities += $entity_count;
        if ($is_unused === 'Unused') {
          // Increment unused bundles counter.
          $unused_bundles++;
        }
        $has_bundles = TRUE;
      }

      if ($has_bundles) {
        $total_entity_types++;
      }
    }
    // Initialize complexity score.
    $complexity_score = $this->entityAnalyzer->calculateComplexityScore(
      $total_entity_types,
      $total_bundles,
      $total_fields,
      $total_entity_references,
      $total_files_image_fields,
      $total_multi_value_fields,
      $total_entities
    );

    // Build the summary table.
    $summary_table = [
      '#type' => 'table',
      '#header' => ['Metric', 'Count'],
      '#rows' => [
        ['Entity Types', $total_entity_types],
        ['Bundles', $total_bundles],
        ['Simple fields', $total_fields],
        ['Entity References & Entity Reference Revisions Fields', $total_entity_references],
        ['File & Image Fields (file, image)', $total_files_image_fields],
        ['Multi-value Fields', $total_multi_value_fields],
        ['Entities', $total_entities],
      ],
      '#attributes' => [
        'class' => ['summary-table'],
      ],
    ];

    return [
      '#theme' => 'migration_complexity_report',
      // Pass grouped data to the Twig template.
      '#grouped_report' => $grouped_report,
      // Include the summary table render array.
      '#summary_table' => $summary_table,
      '#filter_form' => $this->formBuilder->getForm('Drupal\migration_complexity_estimator\Form\MigrationComplexityFilterForm'),
      '#complexity_score' => $complexity_score,
      '#attached' => [
        'library' => [
          'migration_complexity_estimator/migration_complexity_report',
        ],
      ],
    ];
  }

  /**
   * Returns the Fields Grouping tab content.
   *
   * @return array
   *   A render array for the fields grouping report.
   */
  public function fieldsGrouping(): array {
    $entity_types = $this->entityAnalyzer->getEntityTypesAndBundles();
    // Fetch stored filters from tempstore.
    $tempstore = $this->tempStoreFactory->get('migration_complexity_estimator');
    $filters = $tempstore->get('filters') ?? [];

    // Get the filter value from the stored filters.
    $filter = $filters['group_by'] ?? 'content_type';
    // print_r($grouped_by_fields);exit;
    if ($filter === 'fields') {
      // Group by fields.
      $grouped_by_fields = $this->entityAnalyzer->groupContentTypesByFields($entity_types);
      // print_r($grouped_by_fields);exit;
    }
    else {
      // Default to group by content type.
      $grouped_by_fields = $this->entityAnalyzer->groupContentTypesByLabels($entity_types);
    }

    return [
      '#theme' => 'migration_complexity_fields_report',
      '#title' => $this->t('Fields Grouping'),
      '#grouped_by_fields' => $grouped_by_fields,
      '#filter_form' => $this->formBuilder->getForm('Drupal\migration_complexity_estimator\Form\MigrationComplexityFilterForm'),
      '#group_by' => $filter,
      '#attached' => [
        'library' => [
          'migration_complexity_estimator/migration_complexity_report',
        ],
      ],
    ];
  }

}
