<?php

namespace Drupal\migration_complexity_estimator\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Routing\RouteMatchInterface;

/**
 * Class MigrationComplexityFilterForm.
 *
 * Provides a form for filtering migration complexity reports.
 */
class MigrationComplexityFilterForm extends FormBase {

  /**
   * The temp store factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The route match interface.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Constructs a new MigrationComplexityFilterForm.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The temp store factory.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match interface.
   */
  public function __construct(PrivateTempStoreFactory $temp_store_factory, RouteMatchInterface $route_match) {
    $this->tempStoreFactory = $temp_store_factory;
    $this->routeMatch = $route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('tempstore.private'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'migration_complexity_filter_form';
  }

  /**
   * Build the filter form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state interface.
   *
   * @return array
   *   The built form.
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    // Get stored filters from TempStore.
    $tempstore = $this->tempStoreFactory->get('migration_complexity_estimator');
    $filters = $tempstore->get('filters') ?? [];

    $form['filters'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Filters'),
    ];

    // Get the current route name.
    $route_name = $this->routeMatch->getRouteName();

    // Entity Type filter.
    if ($route_name == 'migration_complexity_estimator.report_overview') {
      $form['filters']['entity_type'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Entity Type'),
        '#default_value' => $filters['entity_type'] ?? '',
        '#attributes' => ['placeholder' => $this->t('Enter entity type')],
      ];

      // Bundle filter.
      $form['filters']['bundle'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Bundle'),
        '#default_value' => $filters['bundle'] ?? '',
        '#attributes' => ['placeholder' => $this->t('Enter bundle')],
      ];

      // Label filter.
      $form['filters']['label'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Label'),
        '#default_value' => $filters['label'] ?? '',
        '#attributes' => ['placeholder' => $this->t('Enter label')],
      ];

      // Dropdown for unused status filter.
      $form['filters']['unused_status'] = [
        '#type' => 'select',
        '#title' => $this->t('Filter by Bundle Status'),
        '#options' => [
          'all' => $this->t('All Bundles'),
          'used' => $this->t('Used Bundles'),
          'unused' => $this->t('Unused Bundles'),
        ],
        '#default_value' => $filters['unused_status'] ?? 'all',
      ];
    }

    // Group by filter for fields grouping.
    if ($route_name == 'migration_complexity_estimator.report_fields') {
      $form['filters']['group_by'] = [
        '#type' => 'select',
        '#title' => $this->t('Group by'),
        '#options' => [
          'content_type' => $this->t('Content Type'),
          'fields' => $this->t('Fields'),
        ],
        '#default_value' => $filters['group_by'] ?? 'content_type',
      ];
    }

    // Submit and Reset buttons.
    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
    ];
    $form['actions']['reset'] = [
      '#type' => 'submit',
      '#value' => $this->t('Reset'),
      '#submit' => [[$this, 'resetFilters']],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $filters = $form_state->getValues();

    // Store filters in TempStore.
    $this->tempStoreFactory->get('migration_complexity_estimator')->set('filters', $filters);

    // Redirect to the current route with the filters applied.
    $form_state->setRedirect('<current>');
  }

  /**
   * Reset filters.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state interface.
   */
  public function resetFilters(array &$form, FormStateInterface $form_state): void {
    $this->tempStoreFactory->get('migration_complexity_estimator')->delete('filters');
    $form_state->setRedirect('<current>');
  }

}
