<?php

namespace Drupal\migration_complexity_estimator\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the Migration Complexity Settings form.
 */
class MigrationComplexitySettingsForm extends ConfigFormBase {

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs a new MigrationComplexitySettingsForm.
   *
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(LoggerInterface $logger) {
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('logger.channel.migration_complexity_estimator')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['migration_complexity_estimator.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'migration_complexity_estimator_settings_form';
  }

  /**
   * Build the settings form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state interface.
   *
   * @return array
   *   The built form.
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('migration_complexity_estimator.settings');

    // Define the available complexity factors and their default values.
    $complexity_factors = [
      'entity_types' => ['label' => $this->t('Entity Types'), 'default' => 0.1],
      'bundles' => ['label' => $this->t('Bundles'), 'default' => 0.2],
      'fields' => ['label' => $this->t('Fields'), 'default' => 0.2],
      'entities' => ['label' => $this->t('Entities'), 'default' => 0.1],
      'entity_references' => ['label' => $this->t('ER & ERR Fields'), 'default' => 0.2],
      'files_image_fields' => ['label' => $this->t('File & Image Fields'), 'default' => 0.1],
      'multi_value_fields' => ['label' => $this->t('Multi-value Fields'), 'default' => 0.1],
    ];

    $form['weights'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Migration Complexity Weights'),
      '#description' => $this->t('Adjust the weights for different migration complexity factors. The total must be exactly 1.0.'),
      '#tree' => TRUE,
    ];

    foreach ($complexity_factors as $key => $factor) {
      $default_value = $config->get('weights.' . $key);

      // Ensure the field is not empty (fallback to default if needed)
      if ($default_value === NULL) {
        $default_value = $factor['default'];
      }

      $form['weights'][$key] = [
        '#type' => 'number',
        '#title' => $factor['label'],
        '#default_value' => $default_value,
        '#min' => 0,
        '#max' => 1,
        '#step' => 0.1,
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * Validate form input to ensure total weight is exactly 1.0.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state interface.
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $values = $form_state->getValue('weights', []);
    $total_weight = array_sum($values);

    // Allow small floating point precision differences (e.g., 0.9999 ≈ 1.0)
    if (abs($total_weight - 1.0) > 0.01) {
      $form_state->setErrorByName('weights', $this->t('The total of all weights must be exactly 1.0. Current total: @total', ['@total' => $total_weight]));
    }
  }

  /**
   * Submit handler for the form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state interface.
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->configFactory->getEditable('migration_complexity_estimator.settings');
    $values = $form_state->getValue('weights', []);
    if (!empty($values) && is_array($values)) {
      foreach ($values as $key => $value) {
        if (is_numeric($value)) {
          $config->set('weights.' . $key, (float) $value);
        }
      }

      $config->save();
    }
    else {
      $this->logger->warning('No valid weight values received. Form submission may be incorrect.');
    }

    parent::submitForm($form, $form_state);
  }

}
