<?php

namespace Drupal\migration_complexity_estimator\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;

/**
 * Service for analyzing entity types and bundles.
 */
class EntityAnalyzer {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $bundleInfo;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new EntityAnalyzer.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $bundleInfo
   *   The entity type bundle info service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    EntityFieldManagerInterface $entityFieldManager,
    EntityTypeBundleInfoInterface $bundleInfo,
    ConfigFactoryInterface $configFactory,
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->entityFieldManager = $entityFieldManager;
    $this->bundleInfo = $bundleInfo;
    $this->configFactory = $configFactory;
  }

  /**
   * Gets entity types and their bundles.
   *
   * @return array
   *   An array of entity types and their bundles.
   */
  public function getEntityTypesAndBundles(): array {
    $entity_types = $this->entityTypeManager->getDefinitions();
    $bundle_info = $this->bundleInfo->getAllBundleInfo();

    $entities = [];
    foreach ($entity_types as $entity_type_id => $entity_type) {
      if ($entity_type->getBundleEntityType()) {
        $entities[$entity_type_id] = [
          'label' => $entity_type->getLabel(),
          'bundles' => $bundle_info[$entity_type_id] ?? [],
        ];
      }
    }

    return $entities;
  }

  /**
   * Analyzes fields of a given entity type and bundle.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   * @param string $bundle
   *   The bundle ID.
   *
   * @return array
   *   An array of field information.
   */
  public function analyzeFields(string $entity_type_id, string $bundle): array {
    $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity_type_id, $bundle);

    $field_data = [];
    foreach ($field_definitions as $field_name => $field_definition) {
      if (!$field_definition->getFieldStorageDefinition()->isBaseField()) {
        $field_info = [
          'field_name' => $field_name,
          'field_type' => $field_definition->getType(),
          'required' => $field_definition->isRequired(),
          'cardinality' => $field_definition->getFieldStorageDefinition()->getCardinality(),
        ];

        // Add reference field details.
        if (in_array($field_definition->getType(), ['entity_reference', 'entity_reference_revisions'])) {
          $field_info['is_reference'] = TRUE;
          $field_info['target_type'] = $field_definition->getSetting('target_type') ?? 'N/A';
          $field_info['target_bundles'] = !empty($field_definition->getSetting('handler_settings')['target_bundles'])
            ? $field_definition->getSetting('handler_settings')['target_bundles']
            : ['N/A'];
        }
        else {
          $field_info['is_reference'] = FALSE;
        }

        $field_data[] = $field_info;
      }
    }

    return $field_data;
  }

  /**
   * Counts entities of a given entity type and bundle.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   * @param string $bundle
   *   The bundle ID.
   *
   * @return int
   *   The count of entities.
   */
  public function countEntities(string $entity_type_id, string $bundle): int {
    // Get the entity type definition.
    $entity_type = $this->entityTypeManager->getDefinition($entity_type_id);

    // Start the query.
    $query = $this->entityTypeManager->getStorage($entity_type_id)->getQuery()
      ->accessCheck(TRUE);

    // Check if the entity type has bundles.
    $bundle_field = $entity_type->getKey('bundle');
    if ($bundle_field) {
      // Use the bundle key dynamically.
      $query->condition($bundle_field, $bundle);
    }

    // Execute and return the count.
    return (int) $query->count()->execute();
  }

  /**
   * Formats fields for display.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   * @param string $bundle_id
   *   The bundle ID.
   *
   * @return array
   *   An array of formatted field information.
   */
  public function formatFields(string $entity_type_id, string $bundle_id): array {
    $fields = $this->analyzeFields($entity_type_id, $bundle_id);
    $formatted = [];
    foreach ($fields as $field) {
      if (in_array($field['field_type'], ['entity_reference', 'entity_reference_revisions'])) {
        $target_bundles = !empty($field['target_bundles']) ? implode(', ', $field['target_bundles']) : 'N/A';
        $formatted[] = $field['field_name'] . ' (' . $field['field_type'] . ': ' . $field['target_type'] . ': ' . $target_bundles . ')';
      }
      else {
        $formatted[] = $field['field_name'] . ' (' . $field['field_type'] . ')';
      }
    }
    return $formatted;
  }

  /**
   * Calculates complexity score.
   *
   * @param int $total_entity_types
   *   The total number of entity types.
   * @param int $total_bundles
   *   The total number of bundles.
   * @param int $total_fields
   *   The total number of fields.
   * @param int $total_entity_references
   *   The total number of entity references.
   * @param int $total_files_image_fields
   *   The total number of file and image fields.
   * @param int $total_multi_value_fields
   *   The total number of multi-value fields.
   * @param int $total_entities
   *   The total number of entities.
   *
   * @return array
   *   An array containing the complexity score, label, and class.
   */
  public function calculateComplexityScore(
    int $total_entity_types,
    int $total_bundles,
    int $total_fields,
    int $total_entity_references,
    int $total_files_image_fields,
    int $total_multi_value_fields,
    int $total_entities,
  ): array {
    $config = $this->configFactory->get('migration_complexity_estimator.settings');
    $weights = [
      'entity_types' => $config->get('weights.entity_types') ?? 0.1,
      'bundles' => $config->get('weights.bundles') ?? 0.2,
      'fields' => $config->get('weights.fields') ?? 0.2,
      'entities' => $config->get('weights.entities') ?? 0.1,
      'entity_references' => $config->get('weights.entity_references') ?? 0.2,
      'files_media_fields' => $config->get('weights.files_image_fields') ?? 0.1,
      'multi_value_fields' => $config->get('weights.multi_value_fields') ?? 0.1,
    ];
    $score = (
      ($total_entity_types ?? 0) * $weights['entity_types'] +
      ($total_bundles ?? 0) * $weights['bundles'] +
      ($total_fields ?? 0) * $weights['fields'] +
      ($total_entities ?? 0) * $weights['entities'] +
      ($total_entity_references ?? 0) * $weights['entity_references'] +
      ($total_files_image_fields ?? 0) * $weights['files_media_fields'] +
      ($total_multi_value_fields ?? 0) * $weights['multi_value_fields']
    );

    $percentage_score = min($score / 100, 100);

    if ($percentage_score >= 80) {
      return ['score' => $percentage_score, 'label' => 'High Complexity', 'class' => 'high'];
    }
    elseif ($percentage_score >= 50) {
      return ['score' => $percentage_score, 'label' => 'Medium Complexity', 'class' => 'medium'];
    }
    return ['score' => $percentage_score, 'label' => 'Low Complexity', 'class' => 'low'];
  }

  /**
   * Group content types by labels dynamically.
   *
   * @param array $entity_types
   *   An array of entity types with bundles.
   *
   * @return array
   *   A structured array of grouped content types.
   */
  public function groupContentTypesByLabels(array $entity_types): array {
    $grouped_by_labels = [];
    $merge_candidates = [];

    if (!isset($entity_types['node'])) {
      // Return empty if no node entity types are found.
      return [];
    }

    // Get node entity type.
    $node_entity_type = $entity_types['node'];
    $bundles = $node_entity_type['bundles'];

    foreach ($bundles as $bundle_id => $bundle) {
      if (isset($bundle['status']) && $bundle['status'] === 'Unused') {
        continue;
      }
      // Categorize content types based on labels dynamically.
      if (strpos(strtolower($bundle['label']), 'news') !== FALSE || strpos(strtolower($bundle['label']), 'blog') !== FALSE) {
        $merge_candidates['news_blog'][] = $bundle_id;
      }
      elseif (strpos(strtolower($bundle['label']), 'promo') !== FALSE || strpos(strtolower($bundle['label']), 'campaign') !== FALSE) {
        $merge_candidates['promo'][] = $bundle_id;
      }
      elseif (strpos(strtolower($bundle['label']), 'event') !== FALSE || strpos(strtolower($bundle['label']), 'calendar') !== FALSE) {
        $merge_candidates['event'][] = $bundle_id;
      }
      elseif (strpos(strtolower($bundle['label']), 'service') !== FALSE) {
        $merge_candidates['service'][] = $bundle_id;
      }
      elseif (strpos(strtolower($bundle['label']), 'department') !== FALSE) {
        $merge_candidates['department'][] = $bundle_id;
      }
      elseif (strpos(strtolower($bundle['label']), 'document') !== FALSE) {
        $merge_candidates['document'][] = $bundle_id;
      }
      else {
        $merge_candidates['others'][] = $bundle_id;
      }
    }
    // Compare fields for potential merging between content types in each group.
    foreach ($merge_candidates as $group => $content_types) {
      $grouped_by_labels[$group] = $this->analyzeFieldsForMerging($content_types);
    }

    return $grouped_by_labels;
  }

  /**
   * Groups content types by common and uncommon fields.
   *
   * @param array $entity_types
   *   The entity types and their bundles.
   *
   * @return array
   *   The grouped content types by common and uncommon fields.
   */
  public function groupContentTypesByFields(array $entity_types): array {
    $common_fields = [];
    $uncommon_fields = [];

    if (!isset($entity_types['node'])) {
      // Return empty if no node entity types are found.
      return [];
    }

    // Get node entity type.
    $node_entity_type = $entity_types['node'];
    $bundles = $node_entity_type['bundles'];

    // Collect all fields from all bundles.
    foreach ($bundles as $bundle_id => $bundle) {
      $fields = $this->analyzeFields('node', $bundle_id);

      foreach ($fields as $field) {
        $field_name = $field['field_name'];
        if (!isset($common_fields[$field_name])) {
          $common_fields[$field_name] = 0;
        }
        $common_fields[$field_name]++;
      }
    }

    // Filter out common fields that appear in more than one bundle.
    $common_fields = array_keys(array_filter($common_fields, function ($count) {
      return $count > 1;
    }));

    // Identify uncommon fields for each bundle.
    foreach ($bundles as $bundle_id => $bundle) {
      $fields = $this->analyzeFields('node', $bundle_id);
      $bundle_fields = array_map(function ($field) {
        return $field['field_name'];
      }, $fields);

      $uncommon_fields[$bundle_id] = array_diff($bundle_fields, $common_fields);

    }

    return [
      'common_fields' => $common_fields,
      'uncommon_fields' => $uncommon_fields,
    ];
  }

  /**
   * Helper function to compare fields across content types.
   *
   * @param array $content_types
   *   An array of content type IDs.
   *
   * @return array
   *   An array containing common and uncommon fields.
   */
  public function analyzeFieldsForMerging(array $content_types): array {
    $common_fields = [];
    $all_fields = [];
    $uncommon_fields = [];

    foreach ($content_types as $content_type_id) {
      // Fetch fields for each content type.
      $fields = $this->fetchFieldsForContentType($content_type_id);

      foreach ($fields as $field) {
        // Collecting all fields.
        $all_fields[] = $field['field_name'];
      }
    }

    // Identify common fields.
    $field_counts = array_count_values($all_fields);
    $common_fields = array_filter(array_keys($field_counts), function ($field) use ($field_counts) {
      // Common fields are those that appear more than once.
      return $field_counts[$field] > 1;
    });

    // Now, find uncommon fields.
    foreach ($content_types as $content_type_id) {
      $fields = $this->fetchFieldsForContentType($content_type_id);
      $bundle_fields = array_map(function ($field) {
        // Extract field names.
        return $field['field_name'];
      }, $fields);

      // Uncommon fields are those fields that are in the bundle but not in
      // common fields.
      $uncommon_fields[$content_type_id] = array_diff(
        $bundle_fields,
        $common_fields
      );
    }

    return [
      'common_fields' => $common_fields,
      'uncommon_fields' => $uncommon_fields,
    ];
  }

  /**
   * Fetch fields dynamically for each content type (node).
   *
   * @param string $content_type_id
   *   The content type ID.
   *
   * @return array
   *   An array of field information.
   */
  public function fetchFieldsForContentType(string $content_type_id): array {
    $fields = [];
    $node_type = $this->entityTypeManager->getStorage('node_type')->load($content_type_id);
    if ($node_type) {
      $field_definitions = $this->entityFieldManager->getFieldDefinitions('node', $content_type_id);

      foreach ($field_definitions as $field_name => $field_definition) {
        $fields[] = ['field_name' => $field_name];
      }
    }
    return $fields;
  }

}
